@@ -1,16 +1,16 @@
-/* 
+/*
    Copyright (C) 2010 by Ronnie Sahlberg <ronniesahlberg@gmail.com>
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, see <http://www.gnu.org/licenses/>.
 */
@@ -18,9 +18,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
+#include <inttypes.h>
 #include <string.h>
 #include <poll.h>
 #include <getopt.h>
+#include <unistd.h>
 #include "iscsi.h"
 #include "scsi-lowlevel.h"
 
@@ -79,7 +81,7 @@ void write_cb(struct iscsi_context *iscsi, int status, void *command_data, void
 	fill_read_queue(client);
 
 	if (client->progress) {
-		printf("\r%lu of %lu blocks transferred.", client->pos, client->src_num_blocks);
+		printf("\r%"PRIu64" of %"PRIu64" blocks transferred.", client->pos, client->src_num_blocks);
 	}
 
 	if ((client->in_flight == 0) && (client->pos == client->src_num_blocks)) {
@@ -261,19 +263,12 @@ int main(int argc, char *argv[])
 	}
 	iscsi_url = iscsi_parse_full_url(client.src_iscsi, src_url);
 	if (iscsi_url == NULL) {
-		fprintf(stderr, "Failed to parse URL: %s\n", 
+		fprintf(stderr, "Failed to parse URL: %s\n",
 			iscsi_get_error(client.src_iscsi));
 		exit(10);
 	}
-	iscsi_set_targetname(client.src_iscsi, iscsi_url->target);
 	iscsi_set_session_type(client.src_iscsi, ISCSI_SESSION_NORMAL);
 	iscsi_set_header_digest(client.src_iscsi, ISCSI_HEADER_DIGEST_NONE_CRC32C);
-	if (iscsi_url->user != NULL) {
-		if (iscsi_set_initiator_username_pwd(client.src_iscsi, iscsi_url->user, iscsi_url->passwd) != 0) {
-			fprintf(stderr, "Failed to set initiator username and password\n");
-			exit(10);
-		}
-	}
 	if (iscsi_full_connect_sync(client.src_iscsi, iscsi_url->portal, iscsi_url->lun) != 0) {
 		fprintf(stderr, "Login Failed. %s\n", iscsi_get_error(client.src_iscsi));
 		iscsi_destroy_url(iscsi_url);
@@ -320,19 +315,12 @@ int main(int argc, char *argv[])
 	}
 	iscsi_url = iscsi_parse_full_url(client.dst_iscsi, dst_url);
 	if (iscsi_url == NULL) {
-		fprintf(stderr, "Failed to parse URL: %s\n", 
+		fprintf(stderr, "Failed to parse URL: %s\n",
 			iscsi_get_error(client.dst_iscsi));
 		exit(10);
 	}
-	iscsi_set_targetname(client.dst_iscsi, iscsi_url->target);
 	iscsi_set_session_type(client.dst_iscsi, ISCSI_SESSION_NORMAL);
 	iscsi_set_header_digest(client.dst_iscsi, ISCSI_HEADER_DIGEST_NONE_CRC32C);
-	if (iscsi_url->user != NULL) {
-		if (iscsi_set_initiator_username_pwd(client.dst_iscsi, iscsi_url->user, iscsi_url->passwd) != 0) {
-			fprintf(stderr, "Failed to set initiator username and password\n");
-			exit(10);
-		}
-	}
 	if (iscsi_full_connect_sync(client.dst_iscsi, iscsi_url->portal, iscsi_url->lun) != 0) {
 		fprintf(stderr, "Login Failed. %s\n", iscsi_get_error(client.dst_iscsi));
 		iscsi_destroy_url(iscsi_url);
@@ -378,7 +366,7 @@ int main(int argc, char *argv[])
 	}
 
 	if (client.src_num_blocks > client.dst_num_blocks) {
-		fprintf(stderr, "source LUN is bigger than destination (%lu > %lu sectors)\n", client.src_num_blocks, client.dst_num_blocks);
+		fprintf(stderr, "source LUN is bigger than destination (%"PRIu64" > %"PRIu64" sectors)\n", client.src_num_blocks, client.dst_num_blocks);
 		exit(10);
 	}
 
@@ -390,6 +378,11 @@ int main(int argc, char *argv[])
 		pfd[1].fd = iscsi_get_fd(client.dst_iscsi);
 		pfd[1].events = iscsi_which_events(client.dst_iscsi);
 
+		if (!pfd[0].events && !pfd[1].events) {
+			sleep(1);
+			continue;
+		}
+
 		if (poll(&pfd[0], 2, -1) < 0) {
 			printf("Poll failed");
 			exit(10);
