@@ -42,15 +42,16 @@
 #include "scsi-lowlevel.h"
 
 struct iscsi_sync_state {
-   int finished;
-   int status;
-   struct scsi_task *task;
+        int finished;
+        int status;
+        void *ptr;
+        struct scsi_task *task;
 };
 
 static void
 event_loop(struct iscsi_context *iscsi, struct iscsi_sync_state *state)
 {
-	struct pollfd pfd;
+        struct pollfd pfd;
 	int ret;
 
 	while (state->finished == 0) {
@@ -420,6 +421,26 @@ iscsi_read6_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_read6_iov_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
+		     uint32_t datalen, int blocksize, struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_read6_iov_task(iscsi, lun, lba, datalen, blocksize,
+				 scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Read6 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_read10_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 		  uint32_t datalen, int blocksize,
@@ -442,6 +463,29 @@ iscsi_read10_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_read10_iov_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
+		      uint32_t datalen, int blocksize,
+		      int rdprotect, int dpo, int fua, int fua_nv, int group_number,
+		      struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_read10_iov_task(iscsi, lun, lba, datalen, blocksize, rdprotect,
+				  dpo, fua, fua_nv, group_number,
+				  scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Read10 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_read12_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 		  uint32_t datalen, int blocksize,
@@ -464,6 +508,29 @@ iscsi_read12_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_read12_iov_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
+		      uint32_t datalen, int blocksize,
+		      int rdprotect, int dpo, int fua, int fua_nv, int group_number,
+		      struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_read12_iov_task(iscsi, lun, lba, datalen, blocksize, rdprotect,
+				  dpo, fua, fua_nv, group_number,
+				  scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Read12 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_read16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 		  uint32_t datalen, int blocksize,
@@ -486,6 +553,29 @@ iscsi_read16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_read16_iov_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
+		      uint32_t datalen, int blocksize,
+		      int rdprotect, int dpo, int fua, int fua_nv, int group_number,
+		      struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_read16_iov_task(iscsi, lun, lba, datalen, blocksize, rdprotect,
+				  dpo, fua, fua_nv, group_number,
+				  scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Read16 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_readcapacity10_sync(struct iscsi_context *iscsi, int lun, int lba,
 			  int pmi)
@@ -525,6 +615,55 @@ iscsi_readcapacity16_sync(struct iscsi_context *iscsi, int lun)
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_readdefectdata10_sync(struct iscsi_context *iscsi, int lun,
+                            int req_plist, int req_glist,
+                            int defect_list_format, uint16_t alloc_len)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_readdefectdata10_task(iscsi, lun,
+                                        req_plist, req_glist,
+                                        defect_list_format, alloc_len,
+                                        scsi_sync_cb, &state) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send ReadDefectData10 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
+struct scsi_task *
+iscsi_readdefectdata12_sync(struct iscsi_context *iscsi, int lun,
+                            int req_plist, int req_glist,
+                            int defect_list_format,
+                            uint32_t address_descriptor_index,
+                            uint32_t alloc_len)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_readdefectdata12_task(iscsi, lun,
+                                        req_plist, req_glist,
+                                        defect_list_format,
+                                        address_descriptor_index, alloc_len,
+                                        scsi_sync_cb, &state) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send ReadDefectData12 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_sanitize_sync(struct iscsi_context *iscsi, int lun,
 		    int immed, int ause, int sa, int param_len,
@@ -777,6 +916,29 @@ iscsi_write10_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_write10_iov_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
+		       unsigned char *data, uint32_t datalen, int blocksize,
+		       int wrprotect, int dpo, int fua, int fua_nv, int group_number,
+		       struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_write10_iov_task(iscsi, lun, lba, data, datalen, blocksize,
+				   wrprotect, dpo, fua, fua_nv, group_number,
+				   scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Write10 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_write12_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 		   unsigned char *data, uint32_t datalen, int blocksize,
@@ -800,6 +962,30 @@ iscsi_write12_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_write12_iov_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
+		       unsigned char *data, uint32_t datalen, int blocksize,
+		       int wrprotect, int dpo, int fua, int fua_nv, int group_number,
+		       struct scsi_iovec *iov,int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_write12_iov_task(iscsi, lun, lba,
+				   data, datalen, blocksize, wrprotect,
+				   dpo, fua, fua_nv, group_number,
+				   scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Write12 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_write16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 		   unsigned char *data, uint32_t datalen, int blocksize,
@@ -823,6 +1009,30 @@ iscsi_write16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_write16_iov_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
+		       unsigned char *data, uint32_t datalen, int blocksize,
+		       int wrprotect, int dpo, int fua, int fua_nv, int group_number,
+		       struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_write16_iov_task(iscsi, lun, lba,
+				   data, datalen, blocksize, wrprotect,
+				   dpo, fua, fua_nv, group_number,
+				   scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Write16 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_writeatomic16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 			 unsigned char *data, uint32_t datalen, int blocksize,
@@ -846,6 +1056,30 @@ iscsi_writeatomic16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_writeatomic16_iov_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
+			     unsigned char *data, uint32_t datalen, int blocksize,
+			     int wrprotect, int dpo, int fua, int group_number,
+			     struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_writeatomic16_iov_task(iscsi, lun, lba,
+					 data, datalen, blocksize, wrprotect,
+					 dpo, fua, group_number,
+					 scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send WriteAtomic16 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_orwrite_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 		   unsigned char *data, uint32_t datalen, int blocksize,
@@ -869,6 +1103,30 @@ iscsi_orwrite_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_orwrite_iov_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
+		       unsigned char *data, uint32_t datalen, int blocksize,
+		       int wrprotect, int dpo, int fua, int fua_nv, int group_number,
+		       struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_orwrite_iov_task(iscsi, lun, lba,
+				   data, datalen, blocksize, wrprotect,
+				   dpo, fua, fua_nv, group_number,
+				   scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Orwrite command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_compareandwrite_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 		   unsigned char *data, uint32_t datalen, int blocksize,
@@ -892,6 +1150,30 @@ iscsi_compareandwrite_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_compareandwrite_iov_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
+			       unsigned char *data, uint32_t datalen, int blocksize,
+			       int wrprotect, int dpo, int fua, int fua_nv, int group_number,
+			       struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_compareandwrite_iov_task(iscsi, lun, lba,
+					   data, datalen, blocksize, wrprotect,
+					   dpo, fua, fua_nv, group_number,
+					   scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send CompareAndWrite command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_writeverify10_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 		   unsigned char *data, uint32_t datalen, int blocksize,
@@ -914,6 +1196,29 @@ iscsi_writeverify10_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_writeverify10_iov_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
+			     unsigned char *data, uint32_t datalen, int blocksize,
+			     int wrprotect, int dpo, int bytchk, int group_number,
+			     struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_writeverify10_iov_task(iscsi, lun, lba, data, datalen, blocksize,
+					 wrprotect, dpo, bytchk, group_number,
+					 scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Writeverify10 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_writeverify12_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 		   unsigned char *data, uint32_t datalen, int blocksize,
@@ -937,6 +1242,30 @@ iscsi_writeverify12_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_writeverify12_iov_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
+			 unsigned char *data, uint32_t datalen, int blocksize,
+			 int wrprotect, int dpo, int bytchk, int group_number,
+			 struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_writeverify12_iov_task(iscsi, lun, lba,
+					 data, datalen, blocksize, wrprotect,
+					 dpo, bytchk, group_number,
+					 scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Writeverify12 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_writeverify16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 		   unsigned char *data, uint32_t datalen, int blocksize,
@@ -960,6 +1289,30 @@ iscsi_writeverify16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_writeverify16_iov_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
+			     unsigned char *data, uint32_t datalen, int blocksize,
+			     int wrprotect, int dpo, int bytchk, int group_number,
+			     struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_writeverify16_iov_task(iscsi, lun, lba,
+					 data, datalen, blocksize, wrprotect,
+					 dpo, bytchk, group_number,
+					 scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Writeverify16 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_verify10_sync(struct iscsi_context *iscsi, int lun, unsigned char *data, uint32_t datalen, uint32_t lba,
 		    int vprotect, int dpo, int bytchk, int blocksize)
@@ -980,6 +1333,27 @@ iscsi_verify10_sync(struct iscsi_context *iscsi, int lun, unsigned char *data, u
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_verify10_iov_sync(struct iscsi_context *iscsi, int lun, unsigned char *data, uint32_t datalen, uint32_t lba,
+			int vprotect, int dpo, int bytchk, int blocksize, struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_verify10_iov_task(iscsi, lun, data, datalen, lba, vprotect, dpo, bytchk, blocksize,
+				    scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Verify10 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
+
 struct scsi_task *
 iscsi_verify12_sync(struct iscsi_context *iscsi, int lun, unsigned char *data, uint32_t datalen, uint32_t lba,
 		    int vprotect, int dpo, int bytchk, int blocksize)
@@ -1000,6 +1374,26 @@ iscsi_verify12_sync(struct iscsi_context *iscsi, int lun, unsigned char *data, u
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_verify12_iov_sync(struct iscsi_context *iscsi, int lun, unsigned char *data, uint32_t datalen, uint32_t lba,
+			int vprotect, int dpo, int bytchk, int blocksize, struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_verify12_iov_task(iscsi, lun, data, datalen, lba, vprotect, dpo, bytchk, blocksize,
+				    scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Verify12 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_verify16_sync(struct iscsi_context *iscsi, int lun, unsigned char *data, uint32_t datalen, uint64_t lba,
 		    int vprotect, int dpo, int bytchk, int blocksize)
@@ -1020,6 +1414,26 @@ iscsi_verify16_sync(struct iscsi_context *iscsi, int lun, unsigned char *data, u
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_verify16_iov_sync(struct iscsi_context *iscsi, int lun, unsigned char *data, uint32_t datalen, uint64_t lba,
+			int vprotect, int dpo, int bytchk, int blocksize, struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_verify16_iov_task(iscsi, lun, data, datalen, lba, vprotect, dpo, bytchk, blocksize,
+				    scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send Verify16 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_writesame10_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 		       unsigned char *data, uint32_t datalen,
@@ -1044,6 +1458,31 @@ iscsi_writesame10_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_writesame10_iov_sync(struct iscsi_context *iscsi, int lun, uint32_t lba,
+			   unsigned char *data, uint32_t datalen,
+			   uint16_t num_blocks,
+			   int anchor, int unmap, int wrprotect, int group,
+			   struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_writesame10_iov_task(iscsi, lun, lba,
+				       data, datalen, num_blocks,
+				       anchor, unmap, wrprotect, group,
+				       scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send WRITESAME10 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_writesame16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 		       unsigned char *data, uint32_t datalen,
@@ -1068,6 +1507,31 @@ iscsi_writesame16_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
 	return state.task;
 }
 
+struct scsi_task *
+iscsi_writesame16_iov_sync(struct iscsi_context *iscsi, int lun, uint64_t lba,
+			   unsigned char *data, uint32_t datalen,
+			   uint32_t num_blocks,
+			   int anchor, int unmap, int wrprotect, int group,
+			   struct scsi_iovec *iov, int niov)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_writesame16_iov_task(iscsi, lun, lba,
+				       data, datalen, num_blocks,
+				       anchor, unmap, wrprotect, group,
+				       scsi_sync_cb, &state, iov, niov) == NULL) {
+		iscsi_set_error(iscsi,
+				"Failed to send WRITESAME16 command");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.task;
+}
+
 struct scsi_task *
 iscsi_persistent_reserve_in_sync(struct iscsi_context *iscsi, int lun,
 				 int sa, uint16_t xferlen)
@@ -1305,3 +1769,77 @@ iscsi_modesense10_sync(struct iscsi_context *iscsi, int lun, int llbaa, int dbd,
 
 	return state.task;
 }
+
+void iscsi_free_discovery_data(struct iscsi_context *iscsi _U_,
+                               struct iscsi_discovery_address *da)
+{
+        while (da) {
+                struct iscsi_discovery_address *danext = da->next;
+
+                while (da->portals) {
+                        struct iscsi_target_portal *ponext = da->portals->next;
+                        free(discard_const(da->portals->portal));
+                        free(da->portals);
+                        da->portals = ponext;
+                }
+                free(discard_const(da->target_name));
+                free(da);
+                da = danext;
+        }
+}
+
+static void
+iscsi_discovery_cb(struct iscsi_context *iscsi _U_, int status,
+	      void *command_data, void *private_data)
+{
+	struct iscsi_sync_state *state = private_data;
+        struct iscsi_discovery_address *da;
+        struct iscsi_discovery_address *dahead = NULL;
+        struct iscsi_target_portal *po;
+
+        for (da = command_data; da != NULL; da = da->next) {
+                struct iscsi_discovery_address *datmp;
+
+                datmp = malloc(sizeof(struct iscsi_discovery_address));
+                memset(datmp, 0, sizeof(struct iscsi_discovery_address));
+                datmp->target_name = strdup(da->target_name);
+                datmp->next = dahead;
+                dahead = datmp;
+
+                for (po = da->portals; po != NULL; po = po->next) {
+                        struct iscsi_target_portal *potmp;
+
+                        potmp = malloc(sizeof(struct iscsi_target_portal));
+                        memset(potmp, 0, sizeof(struct iscsi_target_portal));
+                        potmp->portal = strdup(po->portal);
+
+                        potmp->next = dahead->portals;
+                        dahead->portals = potmp;
+                }
+        }
+
+	if (state != NULL) {
+		state->status    = status;
+		state->finished = 1;
+                state->ptr = dahead;
+	}
+}
+
+struct iscsi_discovery_address *
+iscsi_discovery_sync(struct iscsi_context *iscsi)
+{
+	struct iscsi_sync_state state;
+
+	memset(&state, 0, sizeof(state));
+
+	if (iscsi_discovery_async(iscsi, iscsi_discovery_cb, &state) != 0) {
+		iscsi_set_error(iscsi, "Failed to run discovery. %s",
+				iscsi_get_error(iscsi));
+                printf("async discovery call failed\n");
+		return NULL;
+	}
+
+	event_loop(iscsi, &state);
+
+	return state.ptr;
+}
