@@ -75,6 +75,9 @@ static CU_TestInfo tests_compareandwrite[] = {
         { (char *)"Simple", test_compareandwrite_simple },
         { (char *)"DpoFua", test_compareandwrite_dpofua },
         { (char *)"Miscompare", test_compareandwrite_miscompare },
+        { (char *)"Unwritten", test_compareandwrite_unwritten },
+        { (char *)"InvalidDataOutSize",
+          test_compareandwrite_invalid_dataout_size },
         CU_TEST_INFO_NULL
 };
 
@@ -262,6 +265,16 @@ static CU_TestInfo tests_readcapacity16[] = {
         CU_TEST_INFO_NULL
 };
 
+static CU_TestInfo tests_readdefectdata10[] = {
+        { (char *)"Simple", test_readdefectdata10_simple },
+        CU_TEST_INFO_NULL
+};
+
+static CU_TestInfo tests_readdefectdata12[] = {
+        { (char *)"Simple", test_readdefectdata12_simple },
+        CU_TEST_INFO_NULL
+};
+
 static CU_TestInfo tests_readonly[] = {
         { (char *)"ReadOnlySBC", test_readonly_sbc },
         CU_TEST_INFO_NULL
@@ -419,6 +432,8 @@ static CU_TestInfo tests_writesame10[] = {
         { (char *)"UnmapUnaligned", test_writesame10_unmap_unaligned },
         { (char *)"UnmapUntilEnd", test_writesame10_unmap_until_end },
         { (char *)"UnmapVPD", test_writesame10_unmap_vpd },
+        { (char *)"Check", test_writesame10_check },
+        { (char *)"InvalidDataOutSize", test_writesame10_invalid_dataout_size },
         CU_TEST_INFO_NULL
 };
 
@@ -431,6 +446,8 @@ static CU_TestInfo tests_writesame16[] = {
         { (char *)"UnmapUnaligned", test_writesame16_unmap_unaligned },
         { (char *)"UnmapUntilEnd", test_writesame16_unmap_until_end },
         { (char *)"UnmapVPD", test_writesame16_unmap_vpd },
+        { (char *)"Check", test_writesame16_check },
+        { (char *)"InvalidDataOutSize", test_writesame16_invalid_dataout_size },
         CU_TEST_INFO_NULL
 };
 
@@ -509,6 +526,8 @@ static libiscsi_suite_info scsi_suites[] = {
         { "Read16", NON_PGR_FUNCS, tests_read16 },
         { "ReadCapacity10", NON_PGR_FUNCS, tests_readcapacity10 },
         { "ReadCapacity16", NON_PGR_FUNCS, tests_readcapacity16 },
+        { "ReadDefectData10", NON_PGR_FUNCS, tests_readdefectdata10 },
+        { "ReadDefectData12", NON_PGR_FUNCS, tests_readdefectdata12 },
         { "ReadOnly", NON_PGR_FUNCS, tests_readonly },
         { "ReceiveCopyResults", NON_PGR_FUNCS, tests_receive_copy_results },
         { "ReportSupportedOpcodes", NON_PGR_FUNCS,
@@ -561,6 +580,7 @@ static CU_TestInfo tests_iscsi_residuals[] = {
 
 static CU_TestInfo tests_iscsi_tmf[] = {
         { (char *)"AbortTaskSimpleAsync", test_async_abort_simple },
+        { (char *)"LUNResetSimpleAsync", test_async_lu_reset_simple },
         CU_TEST_INFO_NULL
 };
 
@@ -604,6 +624,8 @@ static libiscsi_suite_info all_suites[] = {
         { "Read16", NON_PGR_FUNCS, tests_read16 },
         { "ReadCapacity10", NON_PGR_FUNCS, tests_readcapacity10 },
         { "ReadCapacity16", NON_PGR_FUNCS, tests_readcapacity16 },
+        { "ReadDefectData10", NON_PGR_FUNCS, tests_readdefectdata10 },
+        { "ReadDefectData12", NON_PGR_FUNCS, tests_readdefectdata12 },
         { "ReadOnly", NON_PGR_FUNCS, tests_readonly },
         { "ReceiveCopyResults", NON_PGR_FUNCS, tests_receive_copy_results },
         { "ReportSupportedOpcodes", NON_PGR_FUNCS,
@@ -647,6 +669,8 @@ static libiscsi_suite_info linux_suites[] = {
         { "Read16", NON_PGR_FUNCS, tests_read16 },
         { "ReadCapacity10", NON_PGR_FUNCS, tests_readcapacity10 },
         { "ReadCapacity16", NON_PGR_FUNCS, tests_readcapacity16 },
+        { "ReadDefectData10", NON_PGR_FUNCS, tests_readdefectdata10 },
+        { "ReadDefectData12", NON_PGR_FUNCS, tests_readdefectdata12 },
         { "ReadOnly", NON_PGR_FUNCS, tests_readonly },
         { "ReportSupportedOpcodes", NON_PGR_FUNCS,
           tests_report_supported_opcodes },
@@ -750,6 +774,7 @@ test_setup(void)
 {
         task = NULL;
         read_write_buf = NULL;
+        local_iscsi_queue_pdu = NULL;
 }
 
 void
@@ -993,8 +1018,13 @@ static int connect_scsi_device(struct scsi_device *sdev, const char *initiatorna
                         close(sdev->sgio_fd);
                         return -1;
                 }
+                if (!strncmp(sdev->sgio_dev, "/dev/sg", 7)) {
+                        /* We can not set this until we have the block-size */
+                        printf("Can not use BLKSECTGET for /dev/sg devices\n");
+                        return 0;
+                }
                 if (ioctl(sdev->sgio_fd, BLKSECTGET, &maxsectors) < 0) {
-                        fprintf(stderr, "%s failed to read BLKMAXSECT\n", sdev->sgio_dev);
+                        fprintf(stderr, "%s failed to read BLKSECTGET\n", sdev->sgio_dev);
                         close(sdev->sgio_fd);
                         return -1;
                 }
@@ -1412,6 +1442,17 @@ main(int argc, char *argv[])
                 return -1;
         }
 
+        /* BLKSECTGET for /dev/sg* is a shitshow under linux.
+         * Even 4.2 kernels return number of bytes instead of number
+         * of sectors here. Just force it to 120k and let us get on with
+         * our lives.
+         */
+        if (sd->sgio_dev && !strncmp(sd->sgio_dev, "/dev/sg", 7)) {
+                printf("Looks like a /dev/sg device. Force max iosize "
+                       "to 120k as BLKSECTGET is just broken and can "
+                       "not be used for discovery.\n");
+                maxsectors = 120 * 1024 / block_size;
+        }
         if (maxsectors) {
                 maximum_transfer_length = maxsectors;
                 printf("Bus transfer size is limited to %d blocks. Clamping "
