@@ -51,6 +51,8 @@ void test_teardown(void);
 void test_compareandwrite_simple(void);
 void test_compareandwrite_dpofua(void);
 void test_compareandwrite_miscompare(void);
+void test_compareandwrite_unwritten(void);
+void test_compareandwrite_invalid_dataout_size(void);
 
 void test_extendedcopy_simple(void);
 void test_extendedcopy_param(void);
@@ -168,6 +170,10 @@ void test_readcapacity16_protection(void);
 void test_readcapacity16_simple(void);
 void test_readcapacity16_support(void);
 
+void test_readdefectdata10_simple(void);
+
+void test_readdefectdata12_simple(void);
+
 void test_readonly_sbc(void);
 
 void test_receive_copy_results_copy_status(void);
@@ -272,6 +278,8 @@ void test_writesame10_unmap(void);
 void test_writesame10_unmap_unaligned(void);
 void test_writesame10_unmap_until_end(void);
 void test_writesame10_unmap_vpd(void);
+void test_writesame10_check(void);
+void test_writesame10_invalid_dataout_size(void);
 
 void test_writesame16_simple(void);
 void test_writesame16_beyond_eol(void);
@@ -281,6 +289,8 @@ void test_writesame16_unmap(void);
 void test_writesame16_unmap_unaligned(void);
 void test_writesame16_unmap_until_end(void);
 void test_writesame16_unmap_vpd(void);
+void test_writesame16_check(void);
+void test_writesame16_invalid_dataout_size(void);
 
 void test_writeverify10_simple(void);
 void test_writeverify10_beyond_eol(void);
@@ -312,5 +322,6 @@ void test_multipathio_compareandwrite(void);
 void test_mpio_async_caw(void);
 
 void test_async_abort_simple(void);
+void test_async_lu_reset_simple(void);
 
 #endif        /* _ISCSI_TEST_CU_H_ */
