@@ -1,4 +1,4 @@
-
+/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2013 by Ronnie Sahlberg <ronniesahlberg@gmail.com>
    
@@ -29,12 +29,9 @@
 void
 test_readcapacity10_simple(void)
 {
-	int ret;
+        logging(LOG_VERBOSE, LOG_BLANK_LINE);
+        logging(LOG_VERBOSE, "Test basic READCAPACITY10");
 
-
-	logging(LOG_VERBOSE, LOG_BLANK_LINE);
-	logging(LOG_VERBOSE, "Test basic READCAPACITY10");
-
-	ret = readcapacity10(iscsic, tgt_lun, 0, 0);
-	CU_ASSERT_EQUAL(ret, 0);
+        READCAPACITY10(sd, NULL, 0, 0,
+                       EXPECT_STATUS_GOOD);
 }
