@@ -1,3 +1,4 @@
+/* -*-  mode:c; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil;  -*- */
 /* 
    Copyright (C) 2013 by Ronnie Sahlberg <ronniesahlberg@gmail.com>
    
@@ -28,19 +29,11 @@
 void
 test_report_supported_opcodes_simple(void)
 {
-	int ret;
+        logging(LOG_VERBOSE, LOG_BLANK_LINE);
+        logging(LOG_VERBOSE, "Test basic READ_SUPPORTED_OPCODES");
 
-	logging(LOG_VERBOSE, LOG_BLANK_LINE);
-	logging(LOG_VERBOSE, "Test basic READ_SUPPORTED_OPCODES");
-
-	ret = report_supported_opcodes(iscsic, tgt_lun,
-		0, SCSI_REPORT_SUPPORTING_OPS_ALL, 0, 0,
-		1024, NULL);
-	if (ret == -2) {
-		logging(LOG_NORMAL, "[SKIPPED] READ_SUPPORTED_OPCODES is not "
-			"implemented.");
-		CU_PASS("READ_SUPPORTED_OPCODES is not implemented.");
-		return;
-	}
-	CU_ASSERT_EQUAL(ret, 0);
+        REPORT_SUPPORTED_OPCODES(sd, NULL,
+                                 0, SCSI_REPORT_SUPPORTING_OPS_ALL, 0, 0,
+                                 1024,
+                                 EXPECT_STATUS_GOOD);
 }
